from setuptools import setup

setup(
	name='paystakk',
	version='0.0.4',
	packages=['paystakk'],
	url='',
	license='MIT',
	author='tundebabzy',
	author_email='tundebabzy@gmail.com',
	description='Unofficial Paystacks API wrapper'
)
